/* Removes Captions from the main page. */
article p.caption {
    display: none;
}

/* Styles the 'line block' https://docutils.sourceforge.io/docs/user/rst/quickref.html#line-blocks. */
blockquote {
    background: none;
    border-left-width: 0px;
    padding: 0em;
}

blockquote div.line {
    color: #838383;
    display: inline;
    font-style: normal !important;
    font-size: 150%;
    line-height: 125%;
}

/* Adds the GitHub ribbon. */
#forkongithub a {
    background:rgb(158, 158, 158);
    color:#fff;
    text-decoration:none;
    font-family:arial,sans-serif;
    text-align:center;
    font-weight:bold;
    padding:5px 40px;
    font-size:1rem;
    line-height:2rem;
    position:relative;
    transition:0.5s;
}

#forkongithub a:hover {
    background:#14ca29;
    color:#fff;
}

#forkongithub a::after {
    bottom:1px;
    top:auto;
}

@media screen and (min-width:800px) {
    #forkongithub{
        position:fixed;
        display:block;
        top:0;
        right:0;
        width:200px;
        overflow:hidden;
        height:200px;
        z-index:9999;
    }

    #forkongithub a {
        width:200px;
        position:absolute;
        top:60px;
        right:-60px;
        transform:rotate(45deg);
        -webkit-transform:rotate(45deg);
        -ms-transform:rotate(45deg);
        -moz-transform:rotate(45deg);
        -o-transform:rotate(45deg);
        box-shadow:4px 4px 10px rgba(0,0,0,0.4);
    }
}